#!/usr/bin/env python
#########################################################
#              RETicker v. 0.4.5pre
# a programm to display xml-formatted rss/rdf-feeds in
# a one line display
#########################################################
# please read Help.txt for contact information

#########################################################
# This file contains the classes storing and showing    #
# the actual news                                       #
#########################################################
from __future__ import generators

import rox
from rox import g, tasks

import os
import tool
import gobject

class IconBuffer(gobject.GObject):
	def __init__(self, filename= None, url = None, Stock = g.STOCK_MISSING_IMAGE):
		gobject.GObject.__init__(self)
		self.stock_iconsets = Stock_IconSets()
		self.__filename = filename
		self.__url = url
		self.__stock = Stock
		self.__iconset = None
		self.__pixbuf = None
		self.__pixbuf_active = None
		self.status = None ### Indicates the status: One of None, get_file, get_url or done
		self.t = None
	def get(self):
		if not(self.__iconset): self.__initialize()
		if not(self.__iconset):
			if self.__url: return self.stock_iconsets.get(g.STOCK_CONVERT)
			else: return self.stock_iconsets.get(Stock)
		return self.__iconset
	def __set_image(self):
		#print "pixbuf " + str(self.__pixbuf)
		self.__iconset = g.IconSet(self.__pixbuf)
		tmp_pixbuf = self.__pixbuf.copy()
		tmp_pixbuf.saturate_and_pixelate(tmp_pixbuf, 5, g.FALSE)
		iconsrc = g.IconSource()
		iconsrc.set_pixbuf(tmp_pixbuf)
		iconsrc.set_state_wildcarded(g.FALSE)
		iconsrc.set_state(g.STATE_PRELIGHT)
		self.__iconset.add_source(iconsrc)
		self.emit("icon_changed")
	def reinit(self, filename= None, url = None, Stock = g.STOCK_MISSING_IMAGE):
		self.__filename = filename
		self.__url = url
		self.__stock = Stock
		self.__iconset = None
		self.get()
	def __initialize(self): ### trying to read icon from file
		if self.__filename:
			print "try to read icon from cached file";
			if (self.__set_from_file()): 
				return
		if self.__url:
			print "try to read icon from url";
			self.__set_from_url()
		
	def __set_from_file(self):
		if self.t and self.t.Status() == None: self.t.Kill()
		self.status = 'get_file'
		if os.path.isfile(self.__filename):
			if self.__pixbuf: del self.__pixbuf; self.__pixbuf = None
			try:
				self.__pixbuf = g.gdk.pixbuf_new_from_file(self.__filename)
				self.__set_image()
			except (gobject.GError): return False
		else: return False
		self.status = 'done'; return True
	def __set_from_url(self): tasks.Task(self.__set_from_url_task())
	def __set_from_url_task(self):
		self.status = 'get_url'
		#prog = tool.Win_ProgBar()
		#prog.start_pulse()
		if self.t and self.t.Status() == None: self.t.Kill()
		self.t = tool.urlretrieve(self.__url, self.__filename)
		yield None
		
		### wait a little bit till icon might be cached locally
		from news import HTTP_TIMEOUT
		for tick in range (0, HTTP_TIMEOUT):
			if self.t.Done() == 1: break
			yield tasks.TimeoutBlocker(1);
		#prog.rem()
		if self.t.Status() == None:
			self.t.Kill()
			self.emit("icon_at_url_not_retrieved")
			#rox.alert("Icon could not retrieved from url:\n" + self.__url)
				
		#### try to set from file
		self.__set_from_file()

gobject.signal_new("icon_changed", IconBuffer, gobject.SIGNAL_RUN_LAST,gobject.TYPE_NONE, ())
gobject.signal_new("icon_at_url_not_retrieved", IconBuffer, gobject.SIGNAL_RUN_LAST,gobject.TYPE_NONE, ())

class Stock_IconSets:
	def __init__(self):
		self.__sets = []
	def get(self, Stock):
		for set in self.__sets:
			if set['Stock'] == Stock: return set['iconset']
		new_iconset = self.__create_iconset_from_stock(Stock)
		self.__sets.append({'Stock' : Stock, 'iconset' : new_iconset})
		return new_iconset
	def __create_iconset_from_stock(self, Stock):
		tmp_ebuf = g.EventBox()
		pixbuf = tmp_ebuf.render_icon(Stock, g.ICON_SIZE_SMALL_TOOLBAR, "")
		iconset = g.IconSet(pixbuf)
		tmp_pixbuf = pixbuf.copy()
		tmp_pixbuf.saturate_and_pixelate(tmp_pixbuf, 5, g.FALSE)
		iconsrc = g.IconSource()
		iconsrc.set_pixbuf(tmp_pixbuf)
		iconsrc.set_state_wildcarded(g.FALSE)
		iconsrc.set_state(g.STATE_PRELIGHT)
		iconset.add_source(iconsrc)
		return iconset
